"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.filesWithExtensions = filesWithExtensions;
const Defaults = __importStar(require("./Defaults"));
const readdirp_1 = require("readdirp");
const fs = __importStar(require("node:fs"));
const n_readlines_1 = __importDefault(require("n-readlines"));
const path = __importStar(require("node:path"));
function countFileLines(filePath) {
    const broadbandLines = new n_readlines_1.default(filePath);
    let lineNumber = 1;
    while (broadbandLines.next()) {
        lineNumber++;
    }
    return lineNumber;
}
function dirIsInIgnorePath(options, fullPath, ignorePath) {
    if (path.isAbsolute(ignorePath)) {
        return fullPath.startsWith(ignorePath);
    }
    else {
        const absIgnorePath = path.join(options.src, ignorePath);
        return fullPath.startsWith(absIgnorePath);
    }
}
function fileIsInIgnorePath(options, fullPath, ignorePath) {
    if (path.isAbsolute(ignorePath)) {
        return fullPath == ignorePath;
    }
    else {
        const absIgnorePath = path.join(options.src, ignorePath);
        return fullPath == absIgnorePath;
    }
}
function ignoreDirectory(options, dirName, fullPath) {
    return dirName.startsWith(".") ||
        dirName.startsWith("__") ||
        options["exclude-file"].some((e) => dirIsInIgnorePath(options, fullPath, e)) ||
        options["exclude-regex"]?.test(fullPath) ||
        Defaults.IGNORE_DIRS.includes(dirName.toLowerCase());
}
function isEmscripten(fileWithDir) {
    if (fs.readFileSync(fileWithDir, "utf-8").toString().includes("// EMSCRIPTEN_START_ASM")) {
        console.warn("Parsing", fileWithDir, ":", "File skipped as it contains EMSCRIPTEN code");
        return true;
    }
    return false;
}
function isTooLarge(fileWithDir) {
    if (countFileLines(fileWithDir) > Defaults.MAX_LOC_IN_FILE) {
        console.warn(fileWithDir, "more than", Defaults.MAX_LOC_IN_FILE, "lines of code");
        return true;
    }
    return false;
}
function ignoreFile(options, fileName, fullPath, extensions) {
    return !extensions.some((e) => fileName.endsWith(e)) ||
        fileName.startsWith(".") ||
        fileName.startsWith("__") ||
        Defaults.IGNORE_FILE_PATTERN.test(fileName) ||
        options["exclude-file"].some((e) => fileIsInIgnorePath(options, fullPath, e)) ||
        options["exclude-regex"]?.test(fullPath) ||
        isEmscripten(fullPath) ||
        isTooLarge(fullPath);
}
async function filesWithExtensions(options, extensions) {
    const dir = options.src;
    const files = await (0, readdirp_1.readdirpPromise)(dir, {
        fileFilter: (f) => !ignoreFile(options, f.basename, f.fullPath, extensions),
        directoryFilter: (d) => !ignoreDirectory(options, d.basename, d.fullPath),
        lstat: true
    });
    // @ts-ignore
    return files.map(file => file.fullPath);
}
//# sourceMappingURL=FileUtils.js.map