"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = start;
const Defaults = __importStar(require("./Defaults"));
const FileUtils = __importStar(require("./FileUtils"));
const JsonUtils = __importStar(require("./JsonUtils"));
const VueCodeCleaner = __importStar(require("./VueCodeCleaner"));
const TscUtils = __importStar(require("./TscUtils"));
const babelParser = __importStar(require("@babel/parser"));
const typescript_1 = __importDefault(require("typescript"));
const path = __importStar(require("node:path"));
const fs = __importStar(require("node:fs"));
/**
 * Convert a single JS/TS file to AST
 */
function fileToJsAst(file) {
    try {
        return babelParser.parse(fs.readFileSync(file, "utf-8"), Defaults.BABEL_PARSER_OPTIONS);
    }
    catch {
        return babelParser.parse(fs.readFileSync(file, "utf-8"), Defaults.SAFE_BABEL_PARSER_OPTIONS);
    }
}
/**
 * Convert a single JS/TS code snippet to AST
 */
function codeToJsAst(code) {
    try {
        return babelParser.parse(code, Defaults.BABEL_PARSER_OPTIONS);
    }
    catch {
        return babelParser.parse(code, Defaults.SAFE_BABEL_PARSER_OPTIONS);
    }
}
/**
 * Convert a single vue file to AST
 */
function toVueAst(file) {
    const code = fs.readFileSync(file, "utf-8");
    const cleanedCode = VueCodeCleaner.cleanVueCode(code);
    return codeToJsAst(cleanedCode);
}
function createTsc(srcFiles) {
    try {
        return TscUtils.tscForFiles(srcFiles);
    }
    catch (err) {
        if (err instanceof Error) {
            console.warn("Retrieving types", err.message);
        }
        return undefined;
    }
}
/**
 * Generate AST for JavaScript or TypeScript
 */
async function createJSAst(options) {
    try {
        const srcFiles = await FileUtils.filesWithExtensions(options, Defaults.JS_EXTENSIONS);
        let ts;
        if (options.tsTypes) {
            ts = createTsc(srcFiles);
        }
        for (const file of srcFiles) {
            try {
                const ast = fileToJsAst(file);
                writeAstFile(file, ast, options);
                if (ts) {
                    try {
                        const tsAst = ts.program.getSourceFile(file);
                        typescript_1.default.forEachChild(tsAst, ts.addType);
                        writeTypesFile(file, ts.seenTypes, options);
                        ts.seenTypes.clear();
                    }
                    catch (err) {
                        if (err instanceof Error) {
                            console.warn("Retrieving types", file, ":", err.message);
                        }
                    }
                }
            }
            catch (err) {
                if (err instanceof Error) {
                    console.error("Parsing", file, ":", err.message);
                }
            }
        }
    }
    catch (err) {
        console.error(err);
    }
}
/**
 * Generate AST for .vue files
 */
async function createVueAst(options) {
    const srcFiles = await FileUtils.filesWithExtensions(options, [".vue"]);
    for (const file of srcFiles) {
        try {
            const ast = toVueAst(file);
            if (ast) {
                writeAstFile(file, ast, options);
            }
        }
        catch (err) {
            if (err instanceof Error) {
                console.error(file, err.message);
            }
        }
    }
}
/**
 * Write AST data to a json file
 */
function writeAstFile(file, ast, options) {
    const relativePath = path.relative(options.src, file);
    const outAstFile = path.join(options.output, relativePath + ".json");
    const data = {
        fullName: file,
        relativeName: relativePath,
        ast: ast,
    };
    fs.mkdirSync(path.dirname(outAstFile), { recursive: true });
    fs.writeFileSync(outAstFile, JsonUtils.stringifyCircular(data));
    console.log("Converted AST for", relativePath, "to", outAstFile);
}
/**
 * Write tsc type data to a json file
 */
function writeTypesFile(file, seenTypes, options) {
    const relativePath = path.relative(options.src, file);
    const outTypeFile = path.join(options.output, relativePath + ".typemap");
    fs.mkdirSync(path.dirname(outTypeFile), { recursive: true });
    fs.writeFileSync(outTypeFile, JsonUtils.stringify(Object.fromEntries(seenTypes)));
    console.log("Converted types for", relativePath, "to", outTypeFile);
}
async function createXAst(options) {
    const srcDir = options.src;
    try {
        fs.accessSync(srcDir, fs.constants.R_OK);
    }
    catch (err) {
        console.error(srcDir, "is invalid");
        process.exit(1);
    }
    if (fs.existsSync(path.join(srcDir, "package.json")) ||
        fs.existsSync(path.join(srcDir, "rush.json"))) {
        return await createJSAst(options);
    }
    console.error(srcDir, "unknown project type");
    process.exit(1);
}
/**
 * Method to start the ast generation process
 *
 * @args options CLI arguments
 */
async function start(options) {
    const type = (options.type || "").toLowerCase();
    switch (type) {
        case "nodejs":
        case "js":
        case "javascript":
        case "typescript":
        case "ts":
            return await createJSAst(options);
        case "vue":
            return await createVueAst(options);
        default:
            return await createXAst(options);
    }
}
//# sourceMappingURL=index.js.map